#!/usr/bin/env python3
# coding: utf-8
# File: question_classifier.py
# Author: lhy<lhy_in_blcu@126.com,https://huangyong.github.io>
# Date: 18-10-4

import os
import ahocorasick
from type_enum import word_type_enum, question_type_enum


class QuestionClassifier:
    def __init__(self, g):
        self.g = g
        cur_dir = '/'.join(os.path.abspath(__file__).split('/')[:-1])
        # 　特征词路径
        self.deny_path = os.path.join(cur_dir, 'dict/deny.txt')
        # 加载特征词
        # 1、疾病
        self.disease_wds = self.init_name_by_node('Disease')
        self.department_wds = self.init_name_by_node('Department')
        self.check_wds = self.init_name_by_node('Check')
        self.producer_wds = self.init_name_by_node('Producer')
        self.symptom_wds = self.init_name_by_node('Symptom')

        # 2、食品
        self.food_wds = self.init_name_by_node('Food')
        self.food_material_wds = self.init_name_by_node('FoodMaterial')
        self.food_package_wds = self.init_name_by_node('FoodPackage')
        self.food_process_wds = self.init_name_by_node('FoodProcess')
        self.cooking_style_wds = self.init_name_by_node('CookingStyle')
        self.index_level_wds = self.init_name_by_node('IndexLevel')
        self.index_level_wds += [x[:2] for x in self.index_level_wds]
        self.province_wds = self.init_name_by_node('Province')
        self.nutrient_wds = self.init_name_by_node('Nutrient')

        # 3、运动
        self.exercise_wds = self.init_name_by_node('Exercise')

        # 4、药品
        self.drug_wds = self.init_name_by_node('Drug')

        self.region_words = set(self.department_wds + self.disease_wds + self.check_wds + self.producer_wds + self.symptom_wds
                                + self.food_wds + self.food_material_wds + self.food_package_wds + self.food_process_wds
                                + self.cooking_style_wds + self.index_level_wds + self.province_wds + self.nutrient_wds
                                + self.exercise_wds
                                + self.drug_wds)
        self.deny_words = [i.strip() for i in open(self.deny_path) if i.strip()]
        # 构造领域actree
        self.region_tree = self.build_actree(list(self.region_words))
        # 构建词典
        self.wdtype_dict = self.build_wdtype_dict()

        # 问句疑问词
        # 1、疾病
        self.symptom_qwds = ['症状', '表征', '现象', '症候', '表现']
        self.cause_qwds = ['原因', '成因', '为什么', '怎么会', '怎样才', '咋样才', '怎样会', '如何会', '为啥', '为何', '如何才会', '怎么才会', '会导致', '会造成']
        self.acompany_qwds = ['并发症', '并发', '一起发生', '一并发生', '一起出现', '一并出现', '一同发生', '一同出现', '伴随发生', '伴随', '共现']
        self.food_qwds = ['饮食', '饮用', '吃', '食', '伙食', '膳食', '喝', '菜', '忌口', '补品', '保健品', '食谱', '菜谱', '食用', '食物', '补品']
        self.drug_qwds = ['药', '药品', '用药', '胶囊', '口服液', '炎片']
        self.prevent_qwds = ['预防', '防范', '抵制', '抵御', '防止', '躲避', '逃避', '避开', '免得', '逃开', '避开', '避掉', '躲开', '躲掉', '绕开',
                             '怎样才能不', '怎么才能不', '咋样才能不', '咋才能不', '如何才能不',
                             '怎样才不', '怎么才不', '咋样才不', '咋才不', '如何才不',
                             '怎样才可以不', '怎么才可以不', '咋样才可以不', '咋才可以不', '如何可以不',
                             '怎样才可不', '怎么才可不', '咋样才可不', '咋才可不', '如何可不']
        self.lasttime_qwds = ['周期', '多久', '多长时间', '多少时间', '几天', '几年', '多少天', '多少小时', '几个小时', '多少年']
        self.cureway_qwds = ['怎么治疗', '如何医治', '怎么医治', '怎么治', '怎么医', '如何治', '医治方式', '疗法', '咋治', '怎么办', '咋办', '咋治']
        self.cureprob_qwds = ['多大概率能治好', '多大几率能治好', '治好希望大么', '几率', '几成', '比例', '可能性', '能治', '可治', '可以治', '可以医']
        self.easyget_qwds = ['易感人群', '容易感染', '易发人群', '什么人', '哪些人', '感染', '染上', '得上']
        self.check_qwds = ['检查', '检查项目', '查出', '检查', '测出', '试出']
        self.belong_qwds = ['属于什么科', '属于', '什么科', '科室']
        self.cure_qwds = ['治疗什么', '治啥', '治疗啥', '医治啥', '治愈啥', '主治啥', '主治什么', '有什么用', '有何用', '用处', '用途',
                          '有什么好处', '有什么益处', '有何益处', '用来', '用来做啥', '用来作甚', '需要', '要']

        # 2、食品
        self.crowd_qwds = ['什么人', '啥人', '啥样的人', '哪种人', '人群', '群体', '哪些人']
        self.cooking_specialty_qwds = ['烹饪特点']
        self.food_specialty_qwds = ['膳食特点']
        self.province_qwds = ['地方', '地点', '地', '地域', '地区', '哪里']
        self.unit_qwds = ['单位', '扩展单位', '量词']
        self.content_qwds = ['含有', '有', '包含', '含', '包括', '营养', '元素', '成分']
        self.alias_qwds = ['别名', '化名', '别称', '叫什么', '称为', '近似', '类似', '差不多', '称呼', '菜名', '叫法', '其他']
        self.nature_qwds = ['性质']
        self.cooking_style_qwds = ['哪些菜', '什么菜']
        self.index_level_high_qwds = ['高', '多', '富']
        self.index_level_mid_qwds = ['中']
        self.index_level_low_qwds = ['低', '少']

        print('model init finished ......')

        return

    def init_name_by_node(self, node_name):
        query = f"MATCH (m:{node_name}) return m.name"
        res = self.g.run(query).data()
        return [x['m.name'] for x in res]

    '''分类主函数'''

    def classify(self, question):
        data = {}
        medical_dict = self.check_medical(question)
        if not medical_dict:
            return {}
        data['args'] = medical_dict
        # 收集问句当中所涉及到的实体类型
        types = []
        for type_ in medical_dict.values():
            types += type_

        question_types = []

        # 症状
        if self.check_words(self.symptom_qwds, question) and (word_type_enum.DISEASE in types):
            question_type = question_type_enum.DISEASE_SYMPTOM
            question_types.append(question_type)

        if self.check_words(self.symptom_qwds, question) and (word_type_enum.SYMPTOM in types):
            question_type = question_type_enum.SYMPTOM_DISEASE
            question_types.append(question_type)

        # 原因
        if self.check_words(self.cause_qwds, question) and (word_type_enum.DISEASE in types):
            question_type = question_type_enum.DISEASE_CAUSE
            question_types.append(question_type)
        # 并发症
        if self.check_words(self.acompany_qwds, question) and (word_type_enum.DISEASE in types):
            question_type = question_type_enum.DISEASE_ACOMPANY
            question_types.append(question_type)

        # 推荐食品
        if self.check_words(self.food_qwds, question) and word_type_enum.DISEASE in types:
            deny_status = self.check_words(self.deny_words, question)
            if deny_status:
                question_type = question_type_enum.DISEASE_NOT_FOOD
            else:
                question_type = question_type_enum.DISEASE_DO_FOOD
            question_types.append(question_type)

        # 已知食物找疾病：
        if self.check_words(self.food_qwds + self.cure_qwds, question) and word_type_enum.FOOD in types and word_type_enum.DISEASE in types:
            deny_status = self.check_words(self.deny_words, question)
            if deny_status:
                question_type = question_type_enum.FOOD_NOT_DISEASE
            else:
                question_type = question_type_enum.FOOD_DO_DISEASE
            question_types.append(question_type)

        # 推荐药品
        if self.check_words(self.drug_qwds, question) and word_type_enum.DISEASE in types:
            question_type = question_type_enum.DISEASE_DRUG
            question_types.append(question_type)

        # 药品治啥病
        if self.check_words(self.cure_qwds, question) and word_type_enum.DRUG in types:
            question_type = question_type_enum.DRUG_DISEASE
            question_types.append(question_type)

        # 疾病接受检查项目
        if self.check_words(self.check_qwds, question) and word_type_enum.DISEASE in types:
            question_type = question_type_enum.DISEASE_CHECK
            question_types.append(question_type)

        # 已知检查项目查相应疾病
        if self.check_words(self.check_qwds + self.cure_qwds, question) and word_type_enum.CHECK in types:
            question_type = question_type_enum.CHECK_DISEASE
            question_types.append(question_type)

        # 　症状防御
        if self.check_words(self.prevent_qwds, question) and word_type_enum.DISEASE in types:
            question_type = question_type_enum.DISEASE_PREVENT
            question_types.append(question_type)

        # 疾病医疗周期
        if self.check_words(self.lasttime_qwds, question) and word_type_enum.DISEASE in types:
            question_type = question_type_enum.DISEASE_LASTTIME
            question_types.append(question_type)

        # 疾病治疗方式
        if self.check_words(self.cureway_qwds, question) and word_type_enum.DISEASE in types:
            question_type = question_type_enum.DISEASE_CUREWAY
            question_types.append(question_type)

        # 疾病治愈可能性
        if self.check_words(self.cureprob_qwds, question) and word_type_enum.DISEASE in types:
            question_type = question_type_enum.DISEASE_CUREPROB
            question_types.append(question_type)

        # 疾病易感染人群
        if self.check_words(self.easyget_qwds, question) and word_type_enum.DISEASE in types:
            question_type = question_type_enum.DISEASE_EASYGET
            question_types.append(question_type)

        # 若没有查到相关的外部查询信息，那么则将该疾病的描述信息返回
        if question_types == [] and word_type_enum.DISEASE in types:
            question_types = [question_type_enum.DISEASE_DESC]

        # 若没有查到相关的外部查询信息，那么则将该疾病的描述信息返回
        if question_types == [] and word_type_enum.SYMPTOM in types:
            question_types = [question_type_enum.SYMPTOM_DISEASE]

        # 已知foodMaterial找人群
        if self.check_words(self.crowd_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            deny_status = self.check_words(self.deny_words, question)
            if deny_status:
                question_type = question_type_enum.FOODMATERIAL_NOT_CROWD
            else:
                question_type = question_type_enum.FOODMATERIAL_DO_CROWD
            question_types.append(question_type)

        # 已知foodProcess找人群
        if self.check_words(self.crowd_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            deny_status = self.check_words(self.deny_words, question)
            if deny_status:
                question_type = question_type_enum.FOODPROCESS_NOT_CROWD
            else:
                question_type = question_type_enum.FOODPROCESS_DO_CROWD
            question_types.append(question_type)

        # 食物、含量关系、Level，已知1，2查3
        if self.check_words(self.index_level_high_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_LEVEL_HIGH
            question_types.append(question_type)

        if self.check_words(self.index_level_mid_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_LEVEL_MID
            question_types.append(question_type)

        if self.check_words(self.index_level_low_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_LEVEL_LOW
            question_types.append(question_type)

        if self.check_words(self.index_level_high_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_LEVEL_HIGH
            question_types.append(question_type)

        if self.check_words(self.index_level_mid_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_LEVEL_MID
            question_types.append(question_type)

        if self.check_words(self.index_level_low_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_LEVEL_LOW
            question_types.append(question_type)

        # 食物、含量关系、Level，已知3，2查1
        if self.check_words(self.index_level_high_qwds, question) and word_type_enum.INDEX_LEVEL in types:
            question_type = question_type_enum.INDEX_LEVEL_HIGH
            question_types.append(question_type)

        if self.check_words(self.index_level_mid_qwds, question) and word_type_enum.INDEX_LEVEL in types:
            question_type = question_type_enum.INDEX_LEVEL_MID
            question_types.append(question_type)

        if self.check_words(self.index_level_low_qwds, question) and word_type_enum.INDEX_LEVEL in types:
            question_type = question_type_enum.INDEX_LEVEL_LOW
            question_types.append(question_type)

        # 食物、含量关系、Level，已知1，3查2
        if self.check_words(self.content_qwds + self.index_level_high_qwds + self.index_level_mid_qwds + self.index_level_low_qwds,
                            question) and word_type_enum.INDEX_LEVEL in types and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_LEVEL_CONTENT
            question_types.append(question_type)

        if self.check_words(self.content_qwds + self.index_level_high_qwds + self.index_level_mid_qwds + self.index_level_low_qwds,
                            question) and word_type_enum.INDEX_LEVEL in types and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_LEVEL_CONTENT
            question_types.append(question_type)

        # 已知食物找烹饪特点
        if self.check_words(self.cooking_specialty_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_COOKING_SPECIALTY
            question_types.append(question_type)

        if self.check_words(self.cooking_specialty_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_COOKING_SPECIALTY
            question_types.append(question_type)

        # 已知食物找膳食特点
        if self.check_words(self.food_specialty_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_FOOD_SPECIALTY
            question_types.append(question_type)

        if self.check_words(self.food_specialty_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_FOOD_SPECIALTY
            question_types.append(question_type)

        # 已知食物找地域
        if self.check_words(self.province_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_PROVINCE
            question_types.append(question_type)

        if self.check_words(self.province_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_PROVINCE
            question_types.append(question_type)

        # 已知地域找食物
        if self.check_words(self.food_qwds, question) and word_type_enum.PROVINCE in types:
            question_type = question_type_enum.PROVINCE_FOOD
            question_types.append(question_type)

        # 已知食物查单位
        if self.check_words(self.unit_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_UNIT
            question_types.append(question_type)

        if self.check_words(self.unit_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_UNIT
            question_types.append(question_type)

        # 已知食物查富含的元素
        if self.check_words(self.content_qwds, question) and self.check_words(self.index_level_high_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_HIGH_CONT
            question_types.append(question_type)

        if self.check_words(self.content_qwds, question) and self.check_words(self.index_level_high_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_HIGH_CONT
            question_types.append(question_type)

        # 已知富含的元素查食物
        if self.check_words(self.content_qwds, question) and self.check_words(self.index_level_high_qwds, question) and word_type_enum.NUTRIENT in types:
            question_type = question_type_enum.NUTRIENT_HIGH_CONT
            question_types.append(question_type)

        # 已知食物查低量的元素
        if self.check_words(self.content_qwds, question) and self.check_words(self.index_level_low_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_LOW_CONT
            question_types.append(question_type)

        if self.check_words(self.content_qwds, question) and self.check_words(self.index_level_low_qwds, question) and word_type_enum.FOOD_MATERIAL in types:
            question_type = question_type_enum.FOODMATERIAL_LOW_CONT
            question_types.append(question_type)

        # 已知低量的元素查食物
        if self.check_words(self.content_qwds, question) and self.check_words(self.index_level_low_qwds, question) and word_type_enum.NUTRIENT in types:
            question_type = question_type_enum.NUTRIENT_LOW_CONT
            question_types.append(question_type)

        # 已知菜品查别名
        if self.check_words(self.alias_qwds, question) and word_type_enum.FOOD_PROCESS in types:
            question_type = question_type_enum.FOODPROCESS_ALIAS
            question_types.append(question_type)

        # 将多个分类结果进行合并处理，组装成一个字典
        data['question_types'] = question_types

        return data

    '''构造词对应的类型'''

    def build_wdtype_dict(self):
        wd_dict = dict()
        for wd in self.region_words:
            wd_dict[wd] = []
            if wd in self.disease_wds:
                wd_dict[wd].append(word_type_enum.DISEASE)
            if wd in self.department_wds:
                wd_dict[wd].append(word_type_enum.DEPARTMENT)
            if wd in self.check_wds:
                wd_dict[wd].append(word_type_enum.CHECK)
            if wd in self.symptom_wds:
                wd_dict[wd].append(word_type_enum.SYMPTOM)
            if wd in self.producer_wds:
                wd_dict[wd].append(word_type_enum.PRODUCER)
            if wd in self.food_wds:
                wd_dict[wd].append(word_type_enum.FOOD)
            if wd in self.food_material_wds:
                wd_dict[wd].append(word_type_enum.FOOD_MATERIAL)
            if wd in self.food_package_wds:
                wd_dict[wd].append(word_type_enum.FOOD_PACKAGE)
            if wd in self.food_process_wds:
                wd_dict[wd].append(word_type_enum.FOOD_PROCESS)
            if wd in self.cooking_style_wds:
                wd_dict[wd].append(word_type_enum.COOKING_STYLE)
            if wd in self.index_level_wds:
                wd_dict[wd].append(word_type_enum.INDEX_LEVEL)
            if wd in self.province_wds:
                wd_dict[wd].append(word_type_enum.PROVINCE)
            if wd in self.nutrient_wds:
                wd_dict[wd].append(word_type_enum.NUTRIENT)
            if wd in self.exercise_wds:
                wd_dict[wd].append(word_type_enum.EXERCISE)
            if wd in self.drug_wds:
                wd_dict[wd].append(word_type_enum.DRUG)
        return wd_dict

    '''构造actree，加速过滤'''

    def build_actree(self, wordlist):
        actree = ahocorasick.Automaton()
        for index, word in enumerate(wordlist):
            actree.add_word(word, (index, word))
        actree.make_automaton()
        return actree

    '''问句过滤'''

    def check_medical(self, question):
        region_wds = []
        for i in self.region_tree.iter(question):
            wd = i[1][1]
            region_wds.append(wd)
        stop_wds = []
        for wd1 in region_wds:
            for wd2 in region_wds:
                if wd1 in wd2 and wd1 != wd2:
                    stop_wds.append(wd1)
        final_wds = [i for i in region_wds if i not in stop_wds]
        final_dict = {i: self.wdtype_dict.get(i) for i in final_wds}

        return final_dict

    '''基于特征词进行分类'''

    def check_words(self, wds, sent):
        for wd in wds:
            if wd in sent:
                return True
        return False


if __name__ == '__main__':
    handler = QuestionClassifier()
    while 1:
        question = input('input an question:')
        data = handler.classify(question)
        print(data)
